/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package zulu.morph;

/**
 * Date - 24 Sep 2014
 *
 * @author NkosanaM
 */
public class SuffixAnalyzer {

    private final String LocSuf = "ni";
    private final String DimSuf = "ana";
    private final String VerbTermNeg = "i";
    private final String AugSuf = "azi";
    private final String EvalSuf = "se";
    private final String VerbTerm = "a";
    private String aCase;
    private String CausExt;
    private String tagWord;

    private String removeSuffix(Token aToken) {
        String word = aToken.getWord();
        int len = word.length();
        if (word.endsWith(DimSuf)) {
            word = word.substring(0, len - DimSuf.length());
            setTokenObjects(word, aToken, "[DimSuf]", DimSuf);
           
            return word;
        }

        if (word.endsWith("eni") || word.endsWith("ini")) {
            word = word.substring(0, len - LocSuf.length());
            setTokenObjects(word, aToken, "[LocSuf]", LocSuf);
            return word;
        }
        //case with word ending in "azi"
        if (word.endsWith(AugSuf)) {
            word = word.substring(0, len - AugSuf.length());
            setTokenObjects(word, aToken, "[AugSuf]", AugSuf);
            return word;
        }
        if (word.endsWith(EvalSuf)) {
            word = word.substring(0, len - EvalSuf.length());
            setTokenObjects(word, aToken, "[EvalSuf]", EvalSuf);
            return word;
        }
        //figure out 
        if (word.endsWith(VerbTermNeg) || word.endsWith(VerbTerm)) {
            word = word.substring(0, len - 1);
            if (word.endsWith(VerbTermNeg)) {
                setTokenObjects(word, aToken, "[VerbTermNeg]", VerbTermNeg);
            } else {
                word = word.substring(0, len - VerbTerm.length());
                setTokenObjects(word, aToken, "[VerbTerm]", VerbTerm);
            }
            return word;
        }

        //default case;
        setTokenObjects(word, aToken, "[def]", "def");
        return word;
    }

    private void setTokenObjects(String word, Token aToken, String suffixType, String suffix) {
        aToken.addToQueue(suffixType, suffix);
        aToken.updateWord(word);
        aToken.setSuffixType(suffix);
    }
    
    private void analyseEnd(Token aToken) {
        String word = aToken.getWord();
        int len = word.length();
        switch (aToken.getSuffix()) {
            case LocSuf:
                // possible prefix classes are for 11 and 14 and o/e
                if (word.endsWith("we")) {
                    word = word.substring(0, len - 2);
                    word = word + "o";
                    //add possible noun classes here
                } //vowel coalesce
                else if (word.endsWith("e")) {
                    word = word.substring(0, len - 1);
                    word = word + "a"; 
                } //class dependency
                else if (word.endsWith("wi")) {
                    word = word.substring(0, len - 2);
                    word = word + "u";
                }
                aToken.addToQueue("[NStem]", word);
                break;

            case EvalSuf:
                //consider moving this case or removing it totally
                aToken.addToQueue("[VRoot]", word);
                break;
            case AugSuf: //kazi or azi has evaluative nature
                //consider removing this case too, no dependencies needed
                if (word.endsWith("k")) {
                    word = word.substring(0, len - 1);
                    
                }aToken.addToQueue("[NStem]", word);
                break;
            case DimSuf:
                //ana
                if (word.endsWith("w")) {
                    word = word.substring(0, len - 1) + "u";
                } else if (word.endsWith("y") && !word.endsWith("ny")) {
                    word = word.substring(0, len - 1) + "i";
                    //check for cons
                } else {
                    word = palatizationFix(word);
                }
                aToken.addToQueue("[NStem]", word);
                aToken.updateWord(word);
                break;
            case VerbTermNeg:
                aToken.addToQueue("[VRoot]", word );
                aToken.updateWord(word);
                break;
            case VerbTerm:
                word = palatizationFix(word);

                aToken.addToQueue("[VRoot]", word);
                aToken.updateWord(word);
                break;
            default:
                aToken.addToQueue("[NStem]", word);
                aToken.updateWord(word);
                break;
        }
  
        
    }

    //method analyzes the new end after the suffix has been stripped in order to return words to their normal form
    private String palatizationFix(String word) {
        int len = word.length();
        
        if (word.endsWith("j")) {
            word = word.substring(0, len - 1) + "d";
            return word;
        }

        if (word.endsWith("sh")) {
            word = word.substring(0, len - 2) + "ph";
            return word;
        }

        if (word.endsWith("nj")) {
            word = word.substring(0, len - 2) + "m";
            return word;
        }
        
        if (word.endsWith("njw")) {
            word = word.substring(0, len - 3) + "mb";
            return word;
        }

        if (word.endsWith("tsh")) {
            word = word.substring(0, len - 3) + "t";
            return word;
        }
        if (word.endsWith("ny")) {
            word = word.substring(0, len - 2) + "n";
            return word;
        }

        if (word.endsWith("ntsh")) {
            word = word.substring(0, len - 4) + "mp";
            return word;
        }
        if (word.endsWith("ny")) {
            word = word.substring(0, len - 2) + "mb";
            return word;
        }

        return word;
    }

    public String getCase() {
        return aCase;
    }
    
    public void suffixStem(Token aToken){
        removeSuffix(aToken);
        analyseEnd(aToken);
    }
}
